#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/7/21 15:00
Desc: 期货品种映射表
"""

import re

from akshare.futures import cons


def symbol_varieties(contract_code: str):
    """
    查找到具体合约代码, 返回大写字母的品种名称
    :param contract_code: ru1801
    :return: RU
    """
    symbol_detail = "".join(re.findall(r"\D", contract_code)).upper().strip()
    if symbol_detail == "PTA":
        symbol_detail = "TA"
    return symbol_detail


def symbol_market(symbol_detail: str = "SC"):
    """
    映射出市场代码
    :param symbol_detail:
    :return:
    """
    var_item = symbol_varieties(symbol_detail)
    for market_item, contract_items in cons.market_exchange_symbols.items():
        if var_item in contract_items:
            return market_item


def find_chinese(chinese_string: str):
    """
    查找中文字符
    :param chinese_string: 中文字符串
    :return:
    """
    p = re.compile(r"[\u4e00-\u9fa5]")
    res = re.findall(p, chinese_string)
    return "".join(res)


def chinese_to_english(chinese_var: str):
    """
    映射期货品种中文名称和英文缩写
    :param chinese_var: 期货品种中文名称
    :return: 对应的英文缩写
    """
    chinese_list = [
        "橡胶",
        "天然橡胶",
        "石油沥青",
        "石油沥青(仓库)",
        "石油沥青(厂库)",
        "沥青",
        "沥青仓库",
        "沥青(仓库)",
        "沥青厂库",
        "沥青(厂库)",
        "热轧卷板",
        "热轧卷板厂库",
        "热轧卷板仓库",
        "热轧卷板(厂库)",
        "热轧卷板(仓库)",
        "热轧板卷",
        "燃料油",
        "白银",
        "线材",
        "螺纹钢",
        "螺纹钢(仓库)",
        "螺纹钢(厂库)",
        "铅",
        "铜",
        "铝",
        "锌",
        "黄金",
        "钯金",
        "锡",
        "镍",
        "纸浆",
        "纸浆(仓库)",
        "纸浆(厂库)",
        "豆一",
        "大豆",
        "豆二",
        "胶合板",
        "玉米",
        "玉米淀粉",
        "聚乙烯",
        "LLDPE",
        "LDPE",
        "豆粕",
        "豆油",
        "大豆油",
        "棕榈油",
        "纤维板",
        "鸡蛋",
        "聚氯乙烯",
        "PVC",
        "聚丙烯",
        "PP",
        "焦炭",
        "焦煤",
        "铁矿石",
        "乙二醇",
        "强麦",
        "强筋小麦",
        " 强筋小麦",
        "硬冬白麦",
        "普麦",
        "硬白小麦",
        "硬白小麦（）",
        "皮棉",
        "棉花",
        "一号棉",
        "白糖",
        "PTA",
        "菜籽油",
        "菜油",
        "早籼稻",
        "早籼",
        "甲醇",
        "柴油",
        "玻璃",
        "油菜籽",
        "菜籽",
        "菜籽粕",
        "菜粕",
        "动力煤",
        "粳稻",
        "晚籼稻",
        "晚籼",
        "硅铁",
        "锰硅",
        "硬麦",
        "棉纱",
        "苹果",
        "原油",
        "中质含硫原油",
        "尿素",
        "20号胶",
        "苯乙烯",
        "不锈钢",
        "粳米",
        "20号胶20",
        "红枣",
        "不锈钢仓库",
        "不锈钢厂库",
        "不锈钢(厂库)",
        "不锈钢(仓库)",
        "纯碱",
        "液化石油气",
        "低硫燃料油",
        "纸浆仓库",
        "石油沥青厂库",
        "石油沥青仓库",
        "螺纹钢仓库",
        "螺纹钢厂库",
        "纸浆厂库",
        "低硫燃料油仓库",
        "低硫燃料油厂库",
        "低硫燃料油(仓库)",
        "低硫燃料油(厂库)",
        "短纤",
        "涤纶短纤",
        "生猪",
        "花生",
        "工业硅",
        "氧化铝",
        "丁二烯橡胶",
        "碳酸锂",
        "氧化铝仓库",
        "氧化铝厂库",
        "氧化铝(仓库)",
        "氧化铝(厂库)",
        "烧碱",
        "丁二烯橡胶仓库",
        "丁二烯橡胶厂库",
        "丁二烯橡胶(仓库)",
        "丁二烯橡胶(厂库)",
        "PX",
        "原木",
        "瓶片期货",
        "瓶片",
        "纯苯",
        "多晶硅",
        "铸造铝合金",
        "铜(BC)",
        "胶版印刷纸(仓库)",
        "胶版印刷纸(厂库)",
        "丙烯期货",
        "丙烯",
    ]
    english_list = [
        "RU",
        "RU",
        "BU",
        "BU",
        "BU",
        "BU",
        "BU",
        "BU",
        "BU2",
        "BU2",
        "HC",
        "HC",
        "HC",
        "HC",
        "HC",
        "HC",
        "FU",
        "AG",
        "WR",
        "RB",
        "RB",
        "RB",
        "PB",
        "CU",
        "AL",
        "ZN",
        "AU",
        "AU",
        "SN",
        "NI",
        "SP",
        "SP",
        "SP",
        "A",
        "A",
        "B",
        "BB",
        "C",
        "CS",
        "L",
        "L",
        "L",
        "M",
        "Y",
        "Y",
        "P",
        "FB",
        "JD",
        "V",
        "V",
        "PP",
        "PP",
        "J",
        "JM",
        "I",
        "EG",
        "WH",
        "WH",
        "WH",
        "PM",
        "PM",
        "PM",
        "PM",
        "CF",
        "CF",
        "CF",
        "SR",
        "TA",
        "OI",
        "OI",
        "RI",
        "ER",
        "MA",
        "MA",
        "FG",
        "RS",
        "RS",
        "RM",
        "RM",
        "ZC",
        "JR",
        "LR",
        "LR",
        "SF",
        "SM",
        "WT",
        "CY",
        "AP",
        "SC",
        "SC",
        "UR",
        "NR",
        "EB",
        "SS",
        "RR",
        "NR",
        "CJ",
        "SS",
        "SS",
        "SS",
        "SS",
        "SA",
        "PG",
        "LU",
        "SP",
        "BU",
        "BU",
        "RB",
        "RB",
        "SP",
        "LU",
        "LU",
        "LU",
        "LU",
        "PF",
        "PF",
        "LH",
        "PK",
        "SI",
        "AO",
        "BR",
        "LC",
        "AO",
        "AO",
        "AO",
        "AO",
        "SH",
        "BR",
        "BR",
        "BR",
        "BR",
        "PX",
        "LG",
        "PR",
        "PR",
        "BZ",
        "PS",
        "AD",
        "BC",
        "OP",
        "OP",
        "PL",
        "PL",
    ]
    pos = chinese_list.index(chinese_var)
    return english_list[pos]


if __name__ == "__main__":
    print(chinese_to_english("苹果"))
    symbol = "rb1801"

    var = symbol_varieties("rb1808")
    print(var)

    market = symbol_market("SP")
    print(market)

    chi = find_chinese("a对方水电费dc大V")
    print(chi)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/5/29 23:00
Desc: 百度股市通-A股-财务报表-估值数据
https://gushitong.baidu.com/stock/ab-002044
"""

import pandas as pd
import requests


def stock_zh_valuation_baidu(
    symbol: str = "002044", indicator: str = "总市值", period: str = "近一年"
) -> pd.DataFrame:
    """
    百度股市通-A股-财务报表-估值数据
    https://gushitong.baidu.com/stock/ab-002044
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: choice of {"总市值", "市盈率(TTM)", "市盈率(静)", "市净率", "市现率"}
    :type indicator: str
    :param period: choice of {"近一年", "近三年", "近五年", "近十年", "全部"}
    :type period: str
    :return: 估值数据
    :rtype: pandas.DataFrame
    """
    url = "https://gushitong.baidu.com/opendata"
    params = {
        "openapi": "1",
        "dspName": "iphone",
        "tn": "tangram",
        "client": "app",
        "query": indicator,
        "code": symbol,
        "word": "",
        "resource_id": "51171",
        "market": "ab",
        "tag": indicator,
        "chart_select": period,
        "industry_select": "",
        "skip_industry": "1",
        "finClientType": "pc",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(
        data_json["Result"][0]["DisplayData"]["resultData"]["tplData"]["result"][
            "chartInfo"
        ][0]["body"]
    )
    temp_df.columns = ["date", "value"]
    temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df["value"] = pd.to_numeric(temp_df["value"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_zh_valuation_baidu_df = stock_zh_valuation_baidu(
        symbol="002044", indicator="总市值", period="近一年"
    )
    print(stock_zh_valuation_baidu_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/4/25 20:22
Desc: 乐咕乐股网-赚钱效应分析
https://www.legulegu.com/stockdata/market-activity
"""

from io import StringIO

import pandas as pd
import requests
from bs4 import BeautifulSoup

from akshare.utils.cons import headers


def stock_market_activity_legu() -> pd.DataFrame:
    """
    乐咕乐股网-赚钱效应分析
    https://www.legulegu.com/stockdata/market-activity
    :return: 乐咕乐股网-赚钱效应分析
    :rtype: pandas.DataFrame
    """
    url = "https://legulegu.com/stockdata/market-activity"
    r = requests.get(url, headers=headers)
    temp_df = pd.read_html(StringIO(r.text))[0]
    temp_df_one = temp_df.iloc[:, :2]
    temp_df_one.columns = ["item", "value"]
    temp_df_two = temp_df.iloc[:, 2:4]
    temp_df_two.columns = ["item", "value"]
    temp_df_three = temp_df.iloc[:, 4:6]
    temp_df_three.columns = ["item", "value"]
    temp_df = pd.concat(
        objs=[temp_df_one, temp_df_two, temp_df_three], ignore_index=True
    )
    temp_df.dropna(how="all", axis=0, inplace=True)
    soup = BeautifulSoup(r.text, features="lxml")
    item_str = soup.find(name="div", attrs={"class": "current-index"}).text
    inner_temp_df = pd.DataFrame([item.strip() for item in item_str.split("：")]).T
    inner_temp_df.columns = ["item", "value"]
    temp_df = pd.concat(objs=[temp_df, inner_temp_df], ignore_index=True)
    item_str = soup.find(name="div", attrs={"class": "current-data"}).text.strip()
    inner_temp_df = pd.DataFrame(["统计日期", item_str]).T
    inner_temp_df.columns = ["item", "value"]
    temp_df = pd.concat(objs=[temp_df, inner_temp_df], ignore_index=True)
    temp_df.reset_index(inplace=True, drop=True)
    return temp_df


if __name__ == "__main__":
    stock_market_activity_legu_df = stock_market_activity_legu()
    print(stock_market_activity_legu_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/9/29 16:19
Desc: 巨潮资讯-数据中心-专题统计-公司治理-公司诉讼
http://webapi.cninfo.com.cn/#/thematicStatistics
"""

import time

import pandas as pd
import requests
import py_mini_racer

js_str = """
    function mcode(input) {
                var keyStr = "ABCDEFGHIJKLMNOP" + "QRSTUVWXYZabcdef" + "ghijklmnopqrstuv"   + "wxyz0123456789+/" + "=";
                var output = "";
                var chr1, chr2, chr3 = "";
                var enc1, enc2, enc3, enc4 = "";
                var i = 0;
                do {
                    chr1 = input.charCodeAt(i++);
                    chr2 = input.charCodeAt(i++);
                    chr3 = input.charCodeAt(i++);
                    enc1 = chr1 >> 2;
                    enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
                    enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
                    enc4 = chr3 & 63;
                    if (isNaN(chr2)) {
                        enc3 = enc4 = 64;
                    } else if (isNaN(chr3)) {
                        enc4 = 64;
                    }
                    output = output + keyStr.charAt(enc1) + keyStr.charAt(enc2)
                            + keyStr.charAt(enc3) + keyStr.charAt(enc4);
                    chr1 = chr2 = chr3 = "";
                    enc1 = enc2 = enc3 = enc4 = "";
                } while (i < input.length);

                return output;
            }
"""


def stock_cg_lawsuit_cninfo(
    symbol: str = "全部", start_date: str = "20180630", end_date: str = "20210927"
) -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-公司治理-公司诉讼
    http://webapi.cninfo.com.cn/#/thematicStatistics
    :param symbol: choice of {"全部", "深市主板", "沪市", "创业板", "科创板"}
    :type symbol: str
    :param start_date: 开始统计时间
    :type start_date: str
    :param end_date: 结束统计时间
    :type end_date: str
    :return: 对外担保
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "全部": "",
        "深市主板": "012002",
        "沪市": "012001",
        "创业板": "012015",
        "科创板": "012029",
    }
    url = "http://webapi.cninfo.com.cn/api/sysapi/p_sysapi1055"
    random_time_str = str(int(time.time()))
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(js_str)
    mcode = js_code.call("mcode", random_time_str)
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "mcode": mcode,
        "Origin": "http://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "sdate": "-".join([start_date[:4], start_date[4:6], start_date[6:]]),
        "edate": "-".join([end_date[:4], end_date[4:6], end_date[6:]]),
        "market": symbol_map[symbol],
    }
    r = requests.post(url, headers=headers, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.columns = [
        "公告统计区间",
        "诉讼金额",
        "诉讼次数",
        "证券简称",
        "证券代码",
    ]
    temp_df = temp_df[
        [
            "证券代码",
            "证券简称",
            "公告统计区间",
            "诉讼次数",
            "诉讼金额",
        ]
    ]
    temp_df["诉讼次数"] = pd.to_numeric(temp_df["诉讼次数"])
    temp_df["诉讼金额"] = pd.to_numeric(temp_df["诉讼金额"])
    return temp_df


if __name__ == "__main__":
    stock_cg_lawsuit_cninfo_df = stock_cg_lawsuit_cninfo(
        symbol="全部", start_date="20180928", end_date="20210927"
    )
    print(stock_cg_lawsuit_cninfo_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2026/2/27
Desc: 同花顺-新发基金
https://fund.10jqka.com.cn/datacenter/xfjj/
"""

import json

import pandas as pd
import requests


def fund_new_found_ths(symbol: str = "全部") -> pd.DataFrame:
    """
    同花顺-基金数据-新发基金
    https://fund.10jqka.com.cn/datacenter/xfjj/
    :param symbol: 选择基金类型; choice of {"全部", "发行中", "将发行"}
    :type symbol: str
    :return: 新发基金数据
    :rtype: pandas.DataFrame
    """
    url = "https://fund.10jqka.com.cn/datacenter/xfjj/"
    r = requests.get(url, timeout=15)
    r.encoding = "utf-8"

    # 从页面中提取 jsonData
    # 找到 jsonData= 的位置
    start_idx = r.text.find("jsonData=")
    if start_idx == -1:
        raise ValueError("未找到 jsonData，可能页面结构已变化")

    # 找到第一个 {
    start_bracket = r.text.find("{", start_idx)
    if start_bracket == -1:
        raise ValueError("未找到 JSON 开始括号")

    # 通过计数括号找到完整的JSON对象
    count = 0
    end_idx = start_bracket
    for i in range(start_bracket, len(r.text)):
        if r.text[i] == "{":
            count += 1
        elif r.text[i] == "}":
            count -= 1
            if count == 0:
                end_idx = i + 1
                break

    if end_idx == start_bracket:
        raise ValueError("未找到完整的 JSON 对象")

    json_str = r.text[start_bracket:end_idx]
    data_json = json.loads(json_str)

    # 转换为 DataFrame
    temp_df = pd.DataFrame(data_json).T
    temp_df.reset_index(inplace=True, drop=True)

    # 根据 symbol 筛选数据
    if symbol == "发行中":
        # 发行中: zzfx=1
        temp_df = temp_df[temp_df["zzfx"] == 1]
    elif symbol == "将发行":
        # 将发行: zzfx != 1 (即 buy=0 且起始日在未来)
        temp_df = temp_df[temp_df["zzfx"] != 1]

    # 提取 manager 字段（可能是数组）
    if "manager" in temp_df.columns:
        temp_df["manager"] = temp_df["manager"].apply(
            lambda x: x[0] if isinstance(x, list) and len(x) > 0 else (x if pd.notna(x) else "")
        )

    # 重命名列
    temp_df.rename(
        columns={
            "code": "基金代码",
            "name": "基金名称",
            "type": "投资类型",
            "jjlx": "基金类型",
            "tzfg": "投资风格",
            "start": "募集起始日",
            "end": "募集终止日",
            "orgname": "管理人",
            "manager": "基金经理",
            "zgrgfl": "认购费率",
            "zdrg": "最低认购",
            "zdje": "认购金额",
            "zzfx": "发行中",
            "buy": "可购买",
        },
        inplace=True,
    )

    # 选择需要的列
    columns_order = [
        "基金代码",
        "基金名称",
        "投资类型",
        "募集起始日",
        "募集终止日",
        "管理人",
        "基金经理",
        "认购费率",
        "最低认购",
        "基金类型",
        "投资风格",
    ]

    # 只保留存在的列
    existing_columns = [col for col in columns_order if col in temp_df.columns]
    temp_df = temp_df[existing_columns]

    # 数据类型转换
    if "募集起始日" in temp_df.columns:
        temp_df["募集起始日"] = pd.to_datetime(temp_df["募集起始日"], errors="coerce").dt.date
    if "募集终止日" in temp_df.columns:
        temp_df["募集终止日"] = pd.to_datetime(temp_df["募集终止日"], errors="coerce").dt.date
    if "认购费率" in temp_df.columns:
        temp_df["认购费率"] = pd.to_numeric(temp_df["认购费率"], errors="coerce")
    if "最低认购" in temp_df.columns:
        temp_df["最低认购"] = pd.to_numeric(temp_df["最低认购"], errors="coerce")

    return temp_df


if __name__ == "__main__":
    # 测试获取全部新发基金
    fund_new_found_ths_df = fund_new_found_ths(symbol="全部")
    print(fund_new_found_ths_df)

    # 测试获取发行中的基金
    fund_new_found_ths_issue_df = fund_new_found_ths(symbol="发行中")
    print(fund_new_found_ths_issue_df)

    # 测试获取将发行的基金
    fund_new_found_ths_future_df = fund_new_found_ths(symbol="将发行")
    print(fund_new_found_ths_future_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/8/10 15:30
Desc: 搜猪-生猪大数据-各省均价实时排行榜
https://www.soozhu.com/price/data/center/
"""

import pandas as pd
import requests
from bs4 import BeautifulSoup


def spot_hog_soozhu() -> pd.DataFrame:
    """
    搜猪-生猪大数据-各省均价实时排行榜
    https://www.soozhu.com/price/data/center/
    :return: 各省均价实时排行榜
    :rtype: pandas.DataFrame
    """
    session = requests.session()
    url = "https://www.soozhu.com/price/data/center/"
    r = session.get(url)
    soup = BeautifulSoup(r.text, features="lxml")
    token = soup.find(name="input", attrs={"name": "csrfmiddlewaretoken"})["value"]
    url = "https://www.soozhu.com/price/data/center/"
    payload = {"act": "mapdata", "csrfmiddlewaretoken": token}
    r = session.post(url, data=payload)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["vlist"])
    price_list = [item[0] for item in temp_df["value"]]
    pct_list = [item[1] for item in temp_df["value"]]
    big_df = pd.DataFrame([temp_df["name"].values, price_list, pct_list]).T
    big_df.columns = ["省份", "价格", "涨跌幅"]
    big_df["价格"] = pd.to_numeric(big_df["价格"], errors="coerce")
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["涨跌幅"] = round(big_df["涨跌幅"], 2)
    return big_df


def spot_hog_year_trend_soozhu() -> pd.DataFrame:
    """
    搜猪-生猪大数据-今年以来全国出栏均价走势
    https://www.soozhu.com/price/data/center/
    :return: 今年以来全国出栏均价走势
    :rtype: pandas.DataFrame
    """
    session = requests.session()
    url = "https://www.soozhu.com/price/data/center/"
    r = session.get(url)
    soup = BeautifulSoup(r.text, features="lxml")
    token = soup.find(name="input", attrs={"name": "csrfmiddlewaretoken"})["value"]
    url = "https://www.soozhu.com/price/data/center/"
    payload = {"act": "yeartrend", "csrfmiddlewaretoken": token}
    r = session.post(url, data=payload)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["nationlist"])
    temp_df.columns = ["日期", "价格"]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["价格"] = pd.to_numeric(temp_df["价格"], errors="coerce")
    temp_df.sort_values(by=["日期"], ignore_index=True, inplace=True)
    return temp_df


def spot_hog_lean_price_soozhu() -> pd.DataFrame:
    """
    搜猪-生猪大数据-全国瘦肉型肉猪
    https://www.soozhu.com/price/data/center/
    :return: 全国瘦肉型肉猪
    :rtype: pandas.DataFrame
    """
    session = requests.session()
    url = "https://www.soozhu.com/price/data/center/"
    r = session.get(url)
    soup = BeautifulSoup(r.text, features="lxml")
    token = soup.find(name="input", attrs={"name": "csrfmiddlewaretoken"})["value"]
    url = "https://www.soozhu.com/price/data/center/"
    payload = {"act": "pricetrend", "indid": "", "csrfmiddlewaretoken": token}
    r = session.post(url, data=payload)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["datalist"])
    temp_df.columns = ["日期", "价格"]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["价格"] = pd.to_numeric(temp_df["价格"], errors="coerce")
    temp_df.sort_values(by=["日期"], ignore_index=True, inplace=True)
    return temp_df


def spot_hog_three_way_soozhu() -> pd.DataFrame:
    """
    搜猪-生猪大数据-全国三元仔猪
    https://www.soozhu.com/price/data/center/
    :return: 全国三元仔猪
    :rtype: pandas.DataFrame
    """
    session = requests.session()
    url = "https://www.soozhu.com/price/data/center/"
    r = session.get(url)
    soup = BeautifulSoup(r.text, features="lxml")
    token = soup.find(name="input", attrs={"name": "csrfmiddlewaretoken"})["value"]
    url = "https://www.soozhu.com/price/data/center/"
    payload = {"act": "pricetrend", "indid": "4", "csrfmiddlewaretoken": token}
    r = session.post(url, data=payload)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["datalist"])
    temp_df.columns = ["日期", "价格"]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["价格"] = pd.to_numeric(temp_df["价格"], errors="coerce")
    temp_df.sort_values(by=["日期"], ignore_index=True, inplace=True)
    return temp_df


def spot_hog_crossbred_soozhu() -> pd.DataFrame:
    """
    搜猪-生猪大数据-全国后备二元母猪
    https://www.soozhu.com/price/data/center/
    :return: 全国后备二元母猪
    :rtype: pandas.DataFrame
    """
    session = requests.session()
    url = "https://www.soozhu.com/price/data/center/"
    r = session.get(url)
    soup = BeautifulSoup(r.text, features="lxml")
    token = soup.find(name="input", attrs={"name": "csrfmiddlewaretoken"})["value"]
    url = "https://www.soozhu.com/price/data/center/"
    payload = {"act": "pricetrend", "indid": "6", "csrfmiddlewaretoken": token}
    r = session.post(url, data=payload)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["datalist"])
    temp_df.columns = ["日期", "价格"]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["价格"] = pd.to_numeric(temp_df["价格"], errors="coerce")
    temp_df.sort_values(by=["日期"], ignore_index=True, inplace=True)
    return temp_df


def spot_corn_price_soozhu() -> pd.DataFrame:
    """
    搜猪-生猪大数据-全国玉米价格走势
    https://www.soozhu.com/price/data/center/
    :return: 全国玉米价格走势
    :rtype: pandas.DataFrame
    """
    session = requests.session()
    url = "https://www.soozhu.com/price/data/center/"
    r = session.get(url)
    soup = BeautifulSoup(r.text, features="lxml")
    token = soup.find(name="input", attrs={"name": "csrfmiddlewaretoken"})["value"]
    url = "https://www.soozhu.com/price/data/center/"
    payload = {"act": "pricetrend", "indid": "8", "csrfmiddlewaretoken": token}
    r = session.post(url, data=payload)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["datalist"])
    temp_df.columns = ["日期", "价格"]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["价格"] = pd.to_numeric(temp_df["价格"], errors="coerce")
    temp_df.sort_values(by=["日期"], ignore_index=True, inplace=True)
    return temp_df


def spot_soybean_price_soozhu() -> pd.DataFrame:
    """
    搜猪-生猪大数据-全国豆粕价格走势
    https://www.soozhu.com/price/data/center/
    :return: 全国豆粕价格走势
    :rtype: pandas.DataFrame
    """
    session = requests.session()
    url = "https://www.soozhu.com/price/data/center/"
    r = session.get(url)
    soup = BeautifulSoup(r.text, features="lxml")
    token = soup.find(name="input", attrs={"name": "csrfmiddlewaretoken"})["value"]
    url = "https://www.soozhu.com/price/data/center/"
    payload = {"act": "pricetrend", "indid": "9", "csrfmiddlewaretoken": token}
    r = session.post(url, data=payload)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["datalist"])
    temp_df.columns = ["日期", "价格"]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["价格"] = pd.to_numeric(temp_df["价格"], errors="coerce")
    temp_df.sort_values(by=["日期"], ignore_index=True, inplace=True)
    return temp_df


def spot_mixed_feed_soozhu() -> pd.DataFrame:
    """
    搜猪-生猪大数据-全国育肥猪合料（含自配料）半月走势
    https://www.soozhu.com/price/data/center/
    :return: 全国育肥猪合料（含自配料）半月走势
    :rtype: pandas.DataFrame
    """
    session = requests.session()
    url = "https://www.soozhu.com/price/data/center/"
    r = session.get(url)
    soup = BeautifulSoup(r.text, features="lxml")
    token = soup.find(name="input", attrs={"name": "csrfmiddlewaretoken"})["value"]
    url = "https://www.soozhu.com/price/data/center/"
    payload = {"act": "pricetrend", "indid": "11", "csrfmiddlewaretoken": token}
    r = session.post(url, data=payload)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["datalist"])
    temp_df.columns = ["日期", "价格"]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["价格"] = pd.to_numeric(temp_df["价格"], errors="coerce")
    temp_df.sort_values(by=["日期"], ignore_index=True, inplace=True)
    return temp_df


if __name__ == "__main__":
    spot_hog_soozhu_df = spot_hog_soozhu()
    print(spot_hog_soozhu_df)

    spot_hog_year_trend_soozhu_df = spot_hog_year_trend_soozhu()
    print(spot_hog_year_trend_soozhu_df)

    spot_hog_lean_price_soozhu_df = spot_hog_lean_price_soozhu()
    print(spot_hog_lean_price_soozhu_df)

    spot_hog_three_way_soozhu_df = spot_hog_three_way_soozhu()
    print(spot_hog_three_way_soozhu_df)

    spot_hog_crossbred_soozhu_df = spot_hog_crossbred_soozhu()
    print(spot_hog_crossbred_soozhu_df)

    spot_corn_price_soozhu_df = spot_corn_price_soozhu()
    print(spot_corn_price_soozhu_df)

    spot_soybean_price_soozhu_df = spot_soybean_price_soozhu()
    print(spot_soybean_price_soozhu_df)

    spot_mixed_feed_soozhu_df = spot_mixed_feed_soozhu()
    print(spot_mixed_feed_soozhu_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/7/30 17:00
Desc: 东方财富网-数据中心-特色数据-期权龙虎榜单
https://data.eastmoney.com/other/qqlhb.html
"""

import pandas as pd
import requests


def option_lhb_em(
    symbol: str = "510050",
    indicator: str = "期权交易情况-认沽交易量",
    trade_date: str = "20220121",
) -> pd.DataFrame:
    """
    东方财富网-数据中心-期货期权-期权龙虎榜单
    https://data.eastmoney.com/other/qqlhb.html
    :param symbol: 期权代码; choice of {"510050", "510300", "159919"}
    :type symbol: str
    :param indicator: 需要获取的指标; choice of {"期权交易情况-认沽交易量","期权持仓情况-认沽持仓量", "期权交易情况-认购交易量", "期权持仓情况-认购持仓量"}
    :type indicator: str
    :param trade_date: 交易日期
    :type trade_date: str
    :return: 期权龙虎榜单
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/get"
    params = {
        "type": "RPT_IF_BILLBOARD_TD",
        "sty": "ALL",
        "filter": f"""(SECURITY_CODE="{symbol}")(TRADE_DATE='{
            "-".join([trade_date[:4], trade_date[4:6], trade_date[6:]])
        }')""",
        "p": "1",
        "pss": "200",
        "source": "IFBILLBOARD",
        "client": "WEB",
        "ut": "b2884a393a59ad64002292a3e90d46a5",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    if indicator == "期权交易情况-认沽交易量":
        temp_df = temp_df.iloc[:7, :]
        temp_df.columns = [
            "交易类型",
            "交易日期",
            "证券代码",
            "标的名称",
            "-",
            "-",
            "机构",
            "名次",
            "交易量",
            "增减",
            "净认沽量",
            "占总交易量比例",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
        ]
        temp_df = temp_df[
            [
                "交易类型",
                "交易日期",
                "证券代码",
                "标的名称",
                "名次",
                "机构",
                "交易量",
                "增减",
                "净认沽量",
                "占总交易量比例",
            ]
        ]
        temp_df["交易日期"] = pd.to_datetime(
            temp_df["交易日期"], errors="coerce"
        ).dt.date
        temp_df["名次"] = pd.to_numeric(temp_df["名次"], errors="coerce")
        temp_df["交易量"] = pd.to_numeric(temp_df["交易量"], errors="coerce")
        temp_df["增减"] = pd.to_numeric(temp_df["增减"], errors="coerce")
        temp_df["净认沽量"] = pd.to_numeric(temp_df["净认沽量"], errors="coerce")
        temp_df["占总交易量比例"] = pd.to_numeric(
            temp_df["占总交易量比例"], errors="coerce"
        )
        temp_df.reset_index(drop=True, inplace=True)
        return temp_df
    elif indicator == "期权持仓情况-认沽持仓量":
        temp_df = temp_df.iloc[7:14, :]
        temp_df.columns = [
            "交易类型",
            "交易日期",
            "证券代码",
            "标的名称",
            "-",
            "-",
            "机构",
            "名次",
            "-",
            "-",
            "-",
            "-",
            "-",
            "持仓量",
            "增减",
            "净持仓量",
            "占总交易量比例",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
        ]
        temp_df = temp_df[
            [
                "交易类型",
                "交易日期",
                "证券代码",
                "标的名称",
                "名次",
                "机构",
                "持仓量",
                "增减",
                "净持仓量",
                "占总交易量比例",
            ]
        ]
        temp_df["交易日期"] = pd.to_datetime(
            temp_df["交易日期"], errors="coerce"
        ).dt.date
        temp_df["名次"] = pd.to_numeric(temp_df["名次"], errors="coerce")
        temp_df["持仓量"] = pd.to_numeric(temp_df["持仓量"], errors="coerce")
        temp_df["增减"] = pd.to_numeric(temp_df["增减"], errors="coerce")
        temp_df["净持仓量"] = pd.to_numeric(temp_df["净持仓量"], errors="coerce")
        temp_df["占总交易量比例"] = pd.to_numeric(
            temp_df["占总交易量比例"], errors="coerce"
        )
        temp_df.reset_index(drop=True, inplace=True)
        return temp_df
    elif indicator == "期权交易情况-认购交易量":
        temp_df = temp_df.iloc[14:21, :]
        temp_df.columns = [
            "交易类型",
            "交易日期",
            "证券代码",
            "标的名称",
            "-",
            "-",
            "机构",
            "名次",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "-",
            "交易量",
            "增减",
            "净交易量",
            "占总交易量比例",
            "-",
            "-",
            "-",
            "-",
        ]
        temp_df = temp_df[
            [
                "交易类型",
                "交易日期",
                "证券代码",
                "标的名称",
                "名次",
                "机构",
                "交易量",
                "增减",
                "净交易量",
                "占总交易量比例",
            ]
        ]
        temp_df["交易日期"] = pd.to_datetime(
            temp_df["交易日期"], errors="coerce"
        ).dt.date
        temp_df["名次"] = pd.to_numeric(temp_df["名次"], errors="coerce")
        temp_df["交易量"] = pd.to_numeric(temp_df["交易量"], errors="coerce")
        temp_df["增减"] = pd.to_numeric(temp_df["增减"], errors="coerce")
        temp_df["净交易量"] = pd.to_numeric(temp_df["净交易量"], errors="coerce")
        temp_df["占总交易量比例"] = pd.to_numeric(
            temp_df["占总交易量比例"], errors="coerce"
        )
        temp_df.reset_index(drop=True, inplace=True)
        return temp_df
    elif indicator == "期权持仓情况-认购持仓量":
        temp_df = temp_df.iloc[21:, :]
        temp_df.rename(
            columns={
                "MEMBER_RANK": "名次",
                "MEMBER_NAME_ABBR": "机构",
                "BUY_POSITION": "持仓量",
                "BUY_POSITION_CHANGE": "增减",
                "NET_BUY_POSITION": "净持仓量",
                "BUY_POSITION_RATIO": "占总交易量比例",
                "TRADE_TYPE": "交易类型",
                "TRADE_DATE": "交易日期",
                "SECURITY_CODE": "证券代码",
                "TARGET_NAME": "标的名称",
            },
            inplace=True,
        )
        temp_df = temp_df[
            [
                "交易类型",
                "交易日期",
                "证券代码",
                "标的名称",
                "名次",
                "机构",
                "持仓量",
                "增减",
                "净持仓量",
                "占总交易量比例",
            ]
        ]
        temp_df["交易日期"] = pd.to_datetime(
            temp_df["交易日期"], errors="coerce"
        ).dt.date
        temp_df["名次"] = pd.to_numeric(temp_df["名次"], errors="coerce")
        temp_df["持仓量"] = pd.to_numeric(temp_df["持仓量"], errors="coerce")
        temp_df["增减"] = pd.to_numeric(temp_df["增减"], errors="coerce")
        temp_df["净持仓量"] = pd.to_numeric(temp_df["净持仓量"], errors="coerce")
        temp_df["占总交易量比例"] = pd.to_numeric(
            temp_df["占总交易量比例"], errors="coerce"
        )
        temp_df.reset_index(drop=True, inplace=True)
        return temp_df


if __name__ == "__main__":
    option_lhb_em_df = option_lhb_em(
        symbol="510300", indicator="期权交易情况-认购交易量", trade_date="20220124"
    )
    print(option_lhb_em_df)

    option_lhb_em_df = option_lhb_em(
        symbol="510300", indicator="期权交易情况-认沽交易量", trade_date="20220124"
    )
    print(option_lhb_em_df)

    option_lhb_em_df = option_lhb_em(
        symbol="159919", indicator="期权持仓情况-认购持仓量", trade_date="20240712"
    )
    print(option_lhb_em_df)

    option_lhb_em_df = option_lhb_em(
        symbol="510300", indicator="期权持仓情况-认沽持仓量", trade_date="20220124"
    )
    print(option_lhb_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/3/20 20:00
Desc: 乐咕乐股-A 股市盈率和市净率
https://legulegu.com/stockdata/shanghaiPE
"""

from datetime import datetime

import pandas as pd
import py_mini_racer
import requests

from akshare.stock_feature.stock_a_indicator import get_cookie_csrf

hash_code = """
function e(n) {
    var e, t, r = "", o = -1, f;
    if (n && n.length) {
        f = n.length;
        while ((o += 1) < f) {
            e = n.charCodeAt(o);
            t = o + 1 < f ? n.charCodeAt(o + 1) : 0;
            if (55296 <= e && e <= 56319 && 56320 <= t && t <= 57343) {
                e = 65536 + ((e & 1023) << 10) + (t & 1023);
                o += 1
            }
            if (e <= 127) {
                r += String.fromCharCode(e)
            } else if (e <= 2047) {
                r += String.fromCharCode(192 | e >>> 6 & 31, 128 | e & 63)
            } else if (e <= 65535) {
                r += String.fromCharCode(224 | e >>> 12 & 15, 128 | e >>> 6 & 63, 128 | e & 63)
            } else if (e <= 2097151) {
                r += String.fromCharCode(240 | e >>> 18 & 7, 128 | e >>> 12 & 63, 128 | e >>> 6 & 63, 128 | e & 63)
            }
        }
    }
    return r
}

function t(n) {
    var e, t, r, o, f, i = [], h;
    e = t = r = o = f = 0;
    if (n && n.length) {
        h = n.length;
        n += "";
        while (e < h) {
            r = n.charCodeAt(e);
            t += 1;
            if (r < 128) {
                i[t] = String.fromCharCode(r);
                e += 1
            } else if (r > 191 && r < 224) {
                o = n.charCodeAt(e + 1);
                i[t] = String.fromCharCode((r & 31) << 6 | o & 63);
                e += 2
            } else {
                o = n.charCodeAt(e + 1);
                f = n.charCodeAt(e + 2);
                i[t] = String.fromCharCode(
                    (r & 15) << 12 | (o & 63) << 6 | f & 63);
                e += 3
            }
        }
    }
    return i.join("")
}

function r(n, e) {
    var t = (n & 65535) + (e & 65535)
        , r = (n >> 16) + (e >> 16) + (t >> 16);
    return r << 16 | t & 65535
}

function o(n, e) {
    return n << e | n >>> 32 - e
}

function f(n, e) {
    var t = e ? "0123456789ABCDEF" : "0123456789abcdef", r = "", o, f = 0, i = n.length;
    for (; f < i; f += 1) {
        o = n.charCodeAt(f);
        r += t.charAt(o >>> 4 & 15) + t.charAt(o & 15)
    }
    return r
}

function i(n) {
    var e, t = n.length, r = "";
    for (e = 0; e < t; e += 1) {
        r += String.fromCharCode(n.charCodeAt(e) & 255, n.charCodeAt(e) >>> 8 & 255)
    }
    return r
}

function h(n) {
    var e, t = n.length, r = "";
    for (e = 0; e < t; e += 1) {
        r += String.fromCharCode(n.charCodeAt(e) >>> 8 & 255, n.charCodeAt(e) & 255)
    }
    return r
}

function u(n) {
    var e, t = n.length * 32, r = "";
    for (e = 0; e < t; e += 8) {
        r += String.fromCharCode(n[e >> 5] >>> 24 - e % 32 & 255)
    }
    return r
}

function a(n) {
    var e, t = n.length * 32, r = "";
    for (e = 0; e < t; e += 8) {
        r += String.fromCharCode(n[e >> 5] >>> e % 32 & 255)
    }
    return r
}

function c(n) {
    var e, t = n.length * 8, r = Array(n.length >> 2), o = r.length;
    for (e = 0; e < o; e += 1) {
        r[e] = 0
    }
    for (e = 0; e < t; e += 8) {
        r[e >> 5] |= (n.charCodeAt(e / 8) & 255) << e % 32
    }
    return r
}

function l(n) {
    var e, t = n.length * 8, r = Array(n.length >> 2), o = r.length;
    for (e = 0; e < o; e += 1) {
        r[e] = 0
    }
    for (e = 0; e < t; e += 8) {
        r[e >> 5] |= (n.charCodeAt(e / 8) & 255) << 24 - e % 32
    }
    return r
}

function D(n, e) {
    var t = e.length, r = Array(), o, f, i, h, u, a, c, l;
    a = Array(Math.ceil(n.length / 2));
    h = a.length;
    for (o = 0; o < h; o += 1) {
        a[o] = n.charCodeAt(o * 2) << 8 | n.charCodeAt(o * 2 + 1)
    }
    while (a.length > 0) {
        u = Array();
        i = 0;
        for (o = 0; o < a.length; o += 1) {
            i = (i << 16) + a[o];
            f = Math.floor(i / t);
            i -= f * t;
            if (u.length > 0 || f > 0) {
                u[u.length] = f
            }
        }
        r[r.length] = i;
        a = u
    }
    c = "";
    for (o = r.length - 1; o >= 0; o--) {
        c += e.charAt(r[o])
    }
    l = Math.ceil(n.length * 8 / (Math.log(e.length) / Math.log(2)));
    for (o = c.length; o < l; o += 1) {
        c = e[0] + c
    }
    return c
}

function B(n, e) {
    var t = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/", r = "", o = n.length, f, i, h;
    e = e || "=";
    for (f = 0; f < o; f += 3) {
        h = n.charCodeAt(f) << 16 | (f + 1 < o ? n.charCodeAt(f + 1) << 8 : 0) | (f + 2 < o ? n.charCodeAt(f + 2) : 0);
        for (i = 0; i < 4; i += 1) {
            if (f * 8 + i * 6 > n.length * 8) {
                r += e
            } else {
                r += t.charAt(h >>> 6 * (3 - i) & 63)
            }
        }
    }
    return r
}

function hex(n) {
    return f(u(n, h), t)
}

function u(n) {
    n = h ? e(n) : n;
    return a(C(c(n), n.length * 8))
}

function l(n, t) {
    var r, o, f, i, u;
    n = h ? e(n) : n;
    t = h ? e(t) : t;
    r = c(n);
    if (r.length > 16) {
        r = C(r, n.length * 8)
    }
    o = Array(16),
        f = Array(16);
    for (u = 0; u < 16; u += 1) {
        o[u] = r[u] ^ 909522486;
        f[u] = r[u] ^ 1549556828
    }
    i = C(o.concat(c(t)), 512 + t.length * 8);
    return a(C(f.concat(i), 512 + 128))
}

function C(n, e) {
    var t, o, f, i, h, u = 1732584193, a = -271733879, c = -1732584194, l = 271733878;
    n[e >> 5] |= 128 << e % 32;
    n[(e + 64 >>> 9 << 4) + 14] = e;
    for (t = 0; t < n.length; t += 16) {
        o = u;
        f = a;
        i = c;
        h = l;
        u = s(u, a, c, l, n[t + 0], 7, -680876936);
        l = s(l, u, a, c, n[t + 1], 12, -389564586);
        c = s(c, l, u, a, n[t + 2], 17, 606105819);
        a = s(a, c, l, u, n[t + 3], 22, -1044525330);
        u = s(u, a, c, l, n[t + 4], 7, -176418897);
        l = s(l, u, a, c, n[t + 5], 12, 1200080426);
        c = s(c, l, u, a, n[t + 6], 17, -1473231341);
        a = s(a, c, l, u, n[t + 7], 22, -45705983);
        u = s(u, a, c, l, n[t + 8], 7, 1770035416);
        l = s(l, u, a, c, n[t + 9], 12, -1958414417);
        c = s(c, l, u, a, n[t + 10], 17, -42063);
        a = s(a, c, l, u, n[t + 11], 22, -1990404162);
        u = s(u, a, c, l, n[t + 12], 7, 1804603682);
        l = s(l, u, a, c, n[t + 13], 12, -40341101);
        c = s(c, l, u, a, n[t + 14], 17, -1502002290);
        a = s(a, c, l, u, n[t + 15], 22, 1236535329);
        u = w(u, a, c, l, n[t + 1], 5, -165796510);
        l = w(l, u, a, c, n[t + 6], 9, -1069501632);
        c = w(c, l, u, a, n[t + 11], 14, 643717713);
        a = w(a, c, l, u, n[t + 0], 20, -373897302);
        u = w(u, a, c, l, n[t + 5], 5, -701558691);
        l = w(l, u, a, c, n[t + 10], 9, 38016083);
        c = w(c, l, u, a, n[t + 15], 14, -660478335);
        a = w(a, c, l, u, n[t + 4], 20, -405537848);
        u = w(u, a, c, l, n[t + 9], 5, 568446438);
        l = w(l, u, a, c, n[t + 14], 9, -1019803690);
        c = w(c, l, u, a, n[t + 3], 14, -187363961);
        a = w(a, c, l, u, n[t + 8], 20, 1163531501);
        u = w(u, a, c, l, n[t + 13], 5, -1444681467);
        l = w(l, u, a, c, n[t + 2], 9, -51403784);
        c = w(c, l, u, a, n[t + 7], 14, 1735328473);
        a = w(a, c, l, u, n[t + 12], 20, -1926607734);
        u = F(u, a, c, l, n[t + 5], 4, -378558);
        l = F(l, u, a, c, n[t + 8], 11, -2022574463);
        c = F(c, l, u, a, n[t + 11], 16, 1839030562);
        a = F(a, c, l, u, n[t + 14], 23, -35309556);
        u = F(u, a, c, l, n[t + 1], 4, -1530992060);
        l = F(l, u, a, c, n[t + 4], 11, 1272893353);
        c = F(c, l, u, a, n[t + 7], 16, -155497632);
        a = F(a, c, l, u, n[t + 10], 23, -1094730640);
        u = F(u, a, c, l, n[t + 13], 4, 681279174);
        l = F(l, u, a, c, n[t + 0], 11, -358537222);
        c = F(c, l, u, a, n[t + 3], 16, -722521979);
        a = F(a, c, l, u, n[t + 6], 23, 76029189);
        u = F(u, a, c, l, n[t + 9], 4, -640364487);
        l = F(l, u, a, c, n[t + 12], 11, -421815835);
        c = F(c, l, u, a, n[t + 15], 16, 530742520);
        a = F(a, c, l, u, n[t + 2], 23, -995338651);
        u = E(u, a, c, l, n[t + 0], 6, -198630844);
        l = E(l, u, a, c, n[t + 7], 10, 1126891415);
        c = E(c, l, u, a, n[t + 14], 15, -1416354905);
        a = E(a, c, l, u, n[t + 5], 21, -57434055);
        u = E(u, a, c, l, n[t + 12], 6, 1700485571);
        l = E(l, u, a, c, n[t + 3], 10, -1894986606);
        c = E(c, l, u, a, n[t + 10], 15, -1051523);
        a = E(a, c, l, u, n[t + 1], 21, -2054922799);
        u = E(u, a, c, l, n[t + 8], 6, 1873313359);
        l = E(l, u, a, c, n[t + 15], 10, -30611744);
        c = E(c, l, u, a, n[t + 6], 15, -1560198380);
        a = E(a, c, l, u, n[t + 13], 21, 1309151649);
        u = E(u, a, c, l, n[t + 4], 6, -145523070);
        l = E(l, u, a, c, n[t + 11], 10, -1120210379);
        c = E(c, l, u, a, n[t + 2], 15, 718787259);
        a = E(a, c, l, u, n[t + 9], 21, -343485551);
        u = r(u, o);
        a = r(a, f);
        c = r(c, i);
        l = r(l, h)
    }
    return Array(u, a, c, l)
}

function A(n, e, t, f, i, h) {
    return r(o(r(r(e, n), r(f, h)), i), t)
}

function s(n, e, t, r, o, f, i) {
    return A(e & t | ~e & r, n, e, o, f, i)
}

function w(n, e, t, r, o, f, i) {
    return A(e & r | t & ~r, n, e, o, f, i)
}

function F(n, e, t, r, o, f, i) {
    return A(e ^ t ^ r, n, e, o, f, i)
}

function E(n, e, t, r, o, f, i) {
    return A(t ^ (e | ~r), n, e, o, f, i)
}
"""


def stock_market_pe_lg(symbol: str = "深证") -> pd.DataFrame:
    """
    乐咕乐股-主板市盈率
    https://legulegu.com/stockdata/shanghaiPE
    :param symbol: choice of {"上证", "深证", "创业板", "科创版"}
    :type symbol: str
    :return: 指定市场的市盈率数据
    :rtype: pandas.DataFrame
    """
    js_functions = py_mini_racer.MiniRacer()
    js_functions.eval(hash_code)
    token = js_functions.call("hex", datetime.now().date().isoformat()).lower()
    if symbol in {"上证", "深证", "创业板"}:
        url = "https://legulegu.com/api/stock-data/market-pe"
        symbol_map = {
            "上证": "1",
            "深证": "2",
            "创业板": "4",
        }
        url_map = {
            "上证": "https://legulegu.com/stockdata/shanghaiPE",
            "深证": "https://legulegu.com/stockdata/shenzhenPE",
            "创业板": "https://legulegu.com/stockdata/cybPE",
        }
        params = {"token": token, "marketId": symbol_map[symbol]}
        r = requests.get(url, params=params, **get_cookie_csrf(url=url_map[symbol]))
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["data"])
        temp_df["date"] = (
            pd.to_datetime(temp_df["date"], utc=True)
            .dt.tz_convert("Asia/Shanghai")
            .dt.date
        )
        temp_df = temp_df[
            [
                "date",
                "close",
                "pe",
            ]
        ]
        temp_df.columns = [
            "日期",
            "指数",
            "平均市盈率",
        ]
        return temp_df
    else:
        url = "https://legulegu.com/api/stockdata/get-ke-chuang-ban-pe"
        params = {"token": token}
        r = requests.get(
            url,
            params=params,
            **get_cookie_csrf(url="https://legulegu.com/stockdata/ke-chuang-ban-pe"),
        )
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["data"])
        temp_df["date"] = (
            pd.to_datetime(temp_df["date"], utc=True)
            .dt.tz_convert("Asia/Shanghai")
            .dt.date
        )
        temp_df = temp_df[
            [
                "date",
                "close",
                "pe",
            ]
        ]
        temp_df.columns = [
            "日期",
            "总市值",
            "市盈率",
        ]
        return temp_df


def stock_index_pe_lg(symbol: str = "沪深300") -> pd.DataFrame:
    """
    乐咕乐股-指数市盈率
    https://legulegu.com/stockdata/sz50-ttm-lyr
    :param symbol: choice of {"上证50", "沪深300", "上证380", "创业板50", "中证500", "上证180", "深证红利", "深证100", "中证1000", "上证红利", "中证100", "中证800"}
    :type symbol: str
    :return: 指定指数的市盈率数据
    :rtype: pandas.DataFrame
    """
    js_functions = py_mini_racer.MiniRacer()
    js_functions.eval(hash_code)
    token = js_functions.call("hex", datetime.now().date().isoformat()).lower()
    symbol_map = {
        "上证50": "000016.SH",
        "沪深300": "000300.SH",
        "上证380": "000009.SH",
        "创业板50": "399673.SZ",
        "中证500": "000905.SH",
        "上证180": "000010.SH",
        "深证红利": "399324.SZ",
        "深证100": "399330.SZ",
        "中证1000": "000852.SH",
        "上证红利": "000015.SH",
        "中证100": "000903.SH",
        "中证800": "000906.SH",
    }
    url = "https://legulegu.com/api/stockdata/index-basic-pe"
    params = {"token": token, "indexCode": symbol_map[symbol]}
    r = requests.get(
        url,
        params=params,
        **get_cookie_csrf(url="https://legulegu.com/stockdata/sz50-ttm-lyr"),
    )
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df["date"] = (
        pd.to_datetime(temp_df["date"], utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    temp_df = temp_df[
        [
            "date",
            "close",
            "lyrPe",
            "addLyrPe",
            "middleLyrPe",
            "ttmPe",
            "addTtmPe",
            "middleTtmPe",
        ]
    ]
    temp_df.columns = [
        "日期",
        "指数",
        "等权静态市盈率",
        "静态市盈率",
        "静态市盈率中位数",
        "等权滚动市盈率",
        "滚动市盈率",
        "滚动市盈率中位数",
    ]
    return temp_df


def stock_market_pb_lg(symbol: str = "上证") -> pd.DataFrame:
    """
    乐咕乐股-主板市净率
    https://legulegu.com/stockdata/shanghaiPB
    :param symbol: choice of {"上证", "深证", "创业板", "科创版"}
    :type symbol: str
    :return: 指定市场的市净率数据
    :rtype: pandas.DataFrame
    """
    js_functions = py_mini_racer.MiniRacer()
    js_functions.eval(hash_code)
    token = js_functions.call("hex", datetime.now().date().isoformat()).lower()
    url = "https://legulegu.com/api/stockdata/index-basic-pb"
    symbol_map = {"上证": "1", "深证": "2", "创业板": "4", "科创版": "7"}
    url_map = {
        "上证": "https://legulegu.com/stockdata/shanghaiPB",
        "深证": "https://legulegu.com/stockdata/shenzhenPB",
        "创业板": "https://legulegu.com/stockdata/cybPB",
        "科创版": "https://legulegu.com/stockdata/ke-chuang-ban-pb",
    }
    params = {"token": token, "indexCode": symbol_map[symbol]}
    r = requests.get(url, params=params, **get_cookie_csrf(url=url_map[symbol]))
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df["date"] = (
        pd.to_datetime(temp_df["date"], utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    temp_df = temp_df[
        [
            "date",
            "close",
            "addPb",
            "pb",
            "middlePb",
        ]
    ]
    temp_df.columns = [
        "日期",
        "指数",
        "市净率",
        "等权市净率",
        "市净率中位数",
    ]
    return temp_df


def stock_index_pb_lg(symbol: str = "上证50") -> pd.DataFrame:
    """
    乐咕乐股-指数市净率
    https://legulegu.com/stockdata/sz50-pb
    :param symbol: choice of {"上证50", "沪深300", "上证380", "创业板50", "中证500", "上证180", "深证红利", "深证100", "中证1000", "上证红利", "中证100", "中证800"}
    :type symbol: str
    :return: 指定指数的市净率数据
    :rtype: pandas.DataFrame
    """
    js_functions = py_mini_racer.MiniRacer()
    js_functions.eval(hash_code)
    token = js_functions.call("hex", datetime.now().date().isoformat()).lower()
    symbol_map = {
        "上证50": "000016.SH",
        "沪深300": "000300.SH",
        "上证380": "000009.SH",
        "创业板50": "399673.SZ",
        "中证500": "000905.SH",
        "上证180": "000010.SH",
        "深证红利": "399324.SZ",
        "深证100": "399330.SZ",
        "中证1000": "000852.SH",
        "上证红利": "000015.SH",
        "中证100": "000903.SH",
        "中证800": "000906.SH",
    }
    url = "https://legulegu.com/api/stockdata/index-basic-pb"
    params = {"token": token, "indexCode": symbol_map[symbol]}
    r = requests.get(
        url,
        params=params,
        **get_cookie_csrf(url="https://legulegu.com/stockdata/zz500-ttm-lyr"),
    )
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df["date"] = (
        pd.to_datetime(temp_df["date"], utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    temp_df = temp_df[
        [
            "date",
            "close",
            "addPb",
            "pb",
            "middlePb",
        ]
    ]
    temp_df.columns = [
        "日期",
        "指数",
        "市净率",
        "等权市净率",
        "市净率中位数",
    ]
    return temp_df


if __name__ == "__main__":
    stock_market_pe_lg_df = stock_market_pe_lg(symbol="上证")
    print(stock_market_pe_lg_df)

    stock_market_pe_lg_df = stock_market_pe_lg(symbol="科创版")
    print(stock_market_pe_lg_df)

    for item in {"上证", "深证", "创业板", "科创版"}:
        stock_market_pe_lg_df = stock_market_pe_lg(symbol=item)
        print(stock_market_pe_lg_df)

    stock_index_pe_lg_df = stock_index_pe_lg(symbol="沪深300")
    print(stock_index_pe_lg_df)

    for item in [
        "上证50",
        "沪深300",
        "上证380",
        "创业板50",
        "中证500",
        "上证180",
        "深证红利",
        "深证100",
        "中证1000",
        "上证红利",
        "中证100",
        "中证800",
    ]:
        stock_index_pe_lg_df = stock_index_pe_lg(symbol=item)
        print(stock_index_pe_lg_df)

    for item in {"上证", "深证", "创业板", "科创版"}:
        stock_market_pb_lg_df = stock_market_pb_lg(symbol=item)
        print(stock_market_pb_lg_df)

    for item in [
        "上证50",
        "沪深300",
        "上证380",
        "创业板50",
        "中证500",
        "上证180",
        "深证红利",
        "深证100",
        "中证1000",
        "上证红利",
        "中证100",
        "中证800",
    ]:
        stock_index_pe_lg_df = stock_index_pb_lg(symbol=item)
        print(stock_index_pe_lg_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/10/30 20:24
Desc: 市盈率, 市净率和股息率查询
https://www.legulegu.com/stocklist
https://www.legulegu.com/s/000001
"""

from datetime import datetime
from hashlib import md5

import pandas as pd
import requests
from bs4 import BeautifulSoup

from akshare.utils.cons import headers


def get_cookie_csrf(url: str = "") -> dict:
    """
    乐咕乐股-主板市盈率
    https://legulegu.com/stockdata/shanghaiPE
    :return: 指定市场的市盈率数据
    :rtype: pandas.DataFrame
    """
    # 创建独立的 session，避免污染全局状态
    session = requests.Session()
    session.headers.update(headers)
    r = session.get(url)
    soup = BeautifulSoup(r.text, features="lxml")
    csrf_tag = soup.find(name="meta", attrs={"name": "_csrf"})
    csrf_token = csrf_tag.attrs["content"]
    # 创建新的 headers
    local_headers = headers.copy()
    local_headers.update({"X-CSRF-Token": csrf_token})
    return {"cookies": r.cookies, "headers": local_headers}


def get_token_lg() -> str:
    """
    生成乐咕的 token
    https://legulegu.com/s/002488
    :return: token
    :rtype: str
    """
    current_date_str = datetime.now().date().isoformat()
    obj = md5()
    obj.update(current_date_str.encode("utf-8"))
    token = obj.hexdigest()
    return token


def stock_hk_indicator_eniu(
    symbol: str = "hk01093", indicator: str = "市盈率"
) -> pd.DataFrame:
    """
    亿牛网-港股指标
    https://eniu.com/gu/hk01093/roe
    :param symbol: 港股代码
    :type symbol: str
    :param indicator: 需要获取的指标, choice of {"港股", "市盈率", "市净率", "股息率", "ROE", "市值"}
    :type indicator: str
    :return: 指定 symbol 和 indicator 的数据
    :rtype: pandas.DataFrame
    """
    if indicator == "港股":
        url = "https://eniu.com/static/data/stock_list.json"
        r = requests.get(url, headers=headers)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json)
        temp_df = temp_df[temp_df["stock_id"].str.contains("hk")]
        temp_df.reset_index(inplace=True, drop=True)
        return temp_df
    if indicator == "市盈率":
        url = f"https://eniu.com/chart/peh/{symbol}"
    elif indicator == "市净率":
        url = f"https://eniu.com/chart/pbh/{symbol}"
    elif indicator == "股息率":
        url = f"https://eniu.com/chart/dvh/{symbol}"
    elif indicator == "ROE":
        url = f"https://eniu.com/chart/roeh/{symbol}"
    else:
        url = f"https://eniu.com/chart/marketvalueh/{symbol}"
    r = requests.get(url, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json)
    return temp_df


if __name__ == "__main__":
    stock_hk_indicator_eniu_df = stock_hk_indicator_eniu(
        symbol="hk01093", indicator="市盈率"
    )
    print(stock_hk_indicator_eniu_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
date: 2022/2/14 20:02
desc: 东方财富网-数据中心-特色数据-机构调研
http://data.eastmoney.com/jgdy/
东方财富网-数据中心-特色数据-机构调研-机构调研统计: http://data.eastmoney.com/jgdy/tj.html
东方财富网-数据中心-特色数据-机构调研-机构调研详细: http://data.eastmoney.com/jgdy/xx.html
"""

import pandas as pd
import requests
from tqdm import tqdm


def stock_jgdy_tj_em(date: str = "20220101") -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-机构调研-机构调研统计
    https://data.eastmoney.com/jgdy/tj.html
    :param date: 开始时间
    :type date: str
    :return: 机构调研统计
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "NOTICE_DATE,SUM,RECEIVE_START_DATE,SECURITY_CODE",
        "sortTypes": "-1,-1,-1,1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_ORG_SURVEYNEW",
        "columns": "ALL",
        "quoteColumns": "f2~01~SECURITY_CODE~CLOSE_PRICE,f3~01~SECURITY_CODE~CHANGE_RATE",
        "source": "WEB",
        "client": "WEB",
        "filter": f"""(NUMBERNEW="1")(IS_SOURCE="1")(NOTICE_DATE>'{"-".join([date[:4], date[4:6], date[6:]])}')""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat([big_df, temp_df])
    big_df.reset_index(inplace=True)
    big_df["index"] = list(range(1, len(big_df) + 1))
    big_df.columns = [
        "序号",
        "_",
        "代码",
        "名称",
        "_",
        "公告日期",
        "接待日期",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "接待地点",
        "_",
        "接待方式",
        "_",
        "接待人员",
        "_",
        "_",
        "_",
        "_",
        "_",
        "接待机构数量",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "最新价",
        "涨跌幅",
    ]
    big_df = big_df[
        [
            "序号",
            "代码",
            "名称",
            "最新价",
            "涨跌幅",
            "接待机构数量",
            "接待方式",
            "接待人员",
            "接待地点",
            "接待日期",
            "公告日期",
        ]
    ]
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["接待机构数量"] = pd.to_numeric(big_df["接待机构数量"], errors="coerce")
    big_df["接待日期"] = pd.to_datetime(big_df["接待日期"], errors="coerce").dt.date
    big_df["公告日期"] = pd.to_datetime(big_df["公告日期"], errors="coerce").dt.date
    return big_df


def stock_jgdy_detail_em(date: str = "20241211") -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-机构调研-机构调研详细
    https://data.eastmoney.com/jgdy/xx.html
    :param date: 开始时间
    :type date: str
    :return: 机构调研详细
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "NOTICE_DATE,RECEIVE_START_DATE,SECURITY_CODE,NUMBERNEW",
        "sortTypes": "-1,-1,1,-1",
        "pageSize": "50",
        "pageNumber": "1",
        "reportName": "RPT_ORG_SURVEY",
        "columns": "SECUCODE,SECURITY_CODE,SECURITY_NAME_ABBR,NOTICE_DATE,RECEIVE_START_DATE,"
        "RECEIVE_OBJECT,RECEIVE_PLACE,RECEIVE_WAY_EXPLAIN,INVESTIGATORS,RECEPTIONIST,ORG_TYPE",
        "quoteColumns": "f2~01~SECURITY_CODE~CLOSE_PRICE,f3~01~SECURITY_CODE~CHANGE_RATE",
        "quoteType": "0",
        "source": "WEB",
        "client": "WEB",
        "filter": f"""(IS_SOURCE="1")(RECEIVE_START_DATE>'{"-".join([date[:4], date[4:6], date[6:]])}')""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat([big_df, temp_df])
    big_df.reset_index(inplace=True)
    big_df["index"] = list(range(1, len(big_df) + 1))
    big_df.columns = [
        "序号",
        "_",
        "代码",
        "名称",
        "公告日期",
        "调研日期",
        "调研机构",
        "接待地点",
        "接待方式",
        "调研人员",
        "接待人员",
        "机构类型",
        "最新价",
        "涨跌幅",
    ]
    big_df = big_df[
        [
            "序号",
            "代码",
            "名称",
            "最新价",
            "涨跌幅",
            "调研机构",
            "机构类型",
            "调研人员",
            "接待方式",
            "接待人员",
            "接待地点",
            "调研日期",
            "公告日期",
        ]
    ]
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["调研日期"] = pd.to_datetime(big_df["调研日期"], errors="coerce").dt.date
    big_df["公告日期"] = pd.to_datetime(big_df["公告日期"], errors="coerce").dt.date
    return big_df


if __name__ == "__main__":
    stock_jgdy_tj_em_df = stock_jgdy_tj_em(date="20180928")
    print(stock_jgdy_tj_em_df)

    stock_jgdy_detail_em_df = stock_jgdy_detail_em(date="20210915")
    print(stock_jgdy_detail_em_df)



# !/usr/bin/env python
"""
Date: 2025/12/31
Desc: HTTP 请求工具函数
"""

import random
import time
from typing import Dict, Tuple

import requests
from requests.adapters import HTTPAdapter


def request_with_retry(
    url: str,
    params: Dict = None,
    timeout: int = 15,
    max_retries: int = 3,
    base_delay: float = 1.0,
    random_delay_range: Tuple[float, float] = (0.5, 1.5),
) -> requests.Response:
    """
    带重试机制的 HTTP GET 请求
    :param url: 请求 URL
    :type url: str
    :param params: 请求参数
    :type params: dict
    :param timeout: 超时时间（秒）
    :type timeout: int
    :param max_retries: 最大重试次数
    :type max_retries: int
    :param base_delay: 基础延迟时间（秒），用于指数退避
    :type base_delay: float
    :param random_delay_range: 随机延迟范围（秒）
    :type random_delay_range: tuple
    :return: Response 对象
    :rtype: requests.Response
    :raises: 最后一次请求的异常
    """
    last_exception = None

    for attempt in range(max_retries):
        try:
            # 每次请求创建新的 Session，避免复用连接
            with requests.Session() as session:
                # 禁用连接池复用
                adapter = HTTPAdapter(pool_connections=1, pool_maxsize=1)
                session.mount("http://", adapter)
                session.mount("https://", adapter)

                response = session.get(url, params=params, timeout=timeout)
                response.raise_for_status()
                return response

        except (requests.RequestException, ValueError) as e:
            last_exception = e

            if attempt < max_retries - 1:
                # 指数退避 + 随机抖动
                delay = base_delay * (2**attempt) + random.uniform(*random_delay_range)
                time.sleep(delay)

    raise last_exception



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/4/4 18:10
Desc: 东方财富-数据中心-年报季报
东方财富-数据中心-年报季报-业绩预告
https://data.eastmoney.com/bbsj/202003/yjyg.html
东方财富-数据中心-年报季报-预约披露时间
https://data.eastmoney.com/bbsj/202003/yysj.html
"""

import pandas as pd
import requests
from tqdm import tqdm


def stock_yjkb_em(date: str = "20211231") -> pd.DataFrame:
    """
    东方财富-数据中心-年报季报-业绩快报
    https://data.eastmoney.com/bbsj/202003/yjkb.html
    :param date: 财报发布日期; choice of {"20200331", "20200630", "20200930", "20201231", ...}; 从 20100331 开始
    :type date: str
    :return: 业绩快报
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter.eastmoney.com/securities/api/data/v1/get"
    params = {
        "sortColumns": "UPDATE_DATE,SECURITY_CODE",
        "sortTypes": "-1,-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_FCI_PERFORMANCEE",
        "columns": "ALL",
        "filter": f"""(SECURITY_TYPE_CODE in ("058001001","058001008"))(TRADE_MARKET_CODE!="069001017")
        (REPORT_DATE='{"-".join([date[:4], date[4:6], date[6:]])}')""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    big_df = pd.DataFrame()
    total_page = data_json["result"]["pages"]
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update(
            {
                "pageNumber": page,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.reset_index(inplace=True)
    big_df["index"] = range(1, len(big_df) + 1)
    big_df.columns = [
        "序号",
        "股票代码",
        "股票简称",
        "市场板块",
        "_",
        "证券类型",
        "_",
        "公告日期",
        "_",
        "每股收益",
        "营业收入-营业收入",
        "营业收入-去年同期",
        "净利润-净利润",
        "净利润-去年同期",
        "每股净资产",
        "净资产收益率",
        "营业收入-同比增长",
        "净利润-同比增长",
        "营业收入-季度环比增长",
        "净利润-季度环比增长",
        "所处行业",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "股票代码",
            "股票简称",
            "每股收益",
            "营业收入-营业收入",
            "营业收入-去年同期",
            "营业收入-同比增长",
            "营业收入-季度环比增长",
            "净利润-净利润",
            "净利润-去年同期",
            "净利润-同比增长",
            "净利润-季度环比增长",
            "每股净资产",
            "净资产收益率",
            "所处行业",
            "公告日期",
        ]
    ]
    big_df["每股收益"] = pd.to_numeric(big_df["每股收益"], errors="coerce")
    big_df["营业收入-营业收入"] = pd.to_numeric(
        big_df["营业收入-营业收入"], errors="coerce"
    )
    big_df["营业收入-去年同期"] = pd.to_numeric(
        big_df["营业收入-去年同期"], errors="coerce"
    )
    big_df["营业收入-同比增长"] = pd.to_numeric(
        big_df["营业收入-同比增长"], errors="coerce"
    )
    big_df["营业收入-季度环比增长"] = pd.to_numeric(
        big_df["营业收入-季度环比增长"], errors="coerce"
    )
    big_df["净利润-净利润"] = pd.to_numeric(big_df["净利润-净利润"], errors="coerce")
    big_df["净利润-去年同期"] = pd.to_numeric(
        big_df["净利润-去年同期"], errors="coerce"
    )
    big_df["净利润-同比增长"] = pd.to_numeric(
        big_df["净利润-同比增长"], errors="coerce"
    )
    big_df["净利润-季度环比增长"] = pd.to_numeric(
        big_df["净利润-季度环比增长"], errors="coerce"
    )
    big_df["每股净资产"] = pd.to_numeric(big_df["每股净资产"], errors="coerce")
    big_df["净资产收益率"] = pd.to_numeric(big_df["净资产收益率"], errors="coerce")
    big_df["净资产收益率"] = pd.to_numeric(big_df["净资产收益率"], errors="coerce")
    big_df["公告日期"] = pd.to_datetime(big_df["公告日期"]).dt.date
    return big_df


def stock_yjyg_em(date: str = "20200331") -> pd.DataFrame:
    """
    东方财富-数据中心-年报季报-业绩预告
    https://data.eastmoney.com/bbsj/202003/yjyg.html
    :param date: 财报发布日期; choice of {"20200331", "20200630", "20200930", "20201231", ...}; 从 20081231 开始
    :type date: str
    :return: 业绩预告
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter.eastmoney.com/securities/api/data/v1/get"
    params = {
        "sortColumns": "NOTICE_DATE,SECURITY_CODE",
        "sortTypes": "-1,-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_PUBLIC_OP_NEWPREDICT",
        "columns": "ALL",
        "filter": f" (REPORT_DATE='{'-'.join([date[:4], date[4:6], date[6:]])}')",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    big_df = pd.DataFrame()
    total_page = data_json["result"]["pages"]
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update(
            {
                "pageNumber": page,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.reset_index(inplace=True)
    big_df["index"] = range(1, len(big_df) + 1)
    big_df.columns = [
        "序号",
        "_",
        "股票代码",
        "股票简称",
        "_",
        "公告日期",
        "报告日期",
        "_",
        "预测指标",
        "_",
        "_",
        "_",
        "_",
        "业绩变动",
        "业绩变动原因",
        "预告类型",
        "上年同期值",
        "_",
        "_",
        "_",
        "_",
        "业绩变动幅度",
        "预测数值",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "股票代码",
            "股票简称",
            "预测指标",
            "业绩变动",
            "预测数值",
            "业绩变动幅度",
            "业绩变动原因",
            "预告类型",
            "上年同期值",
            "公告日期",
        ]
    ]
    big_df["公告日期"] = pd.to_datetime(big_df["公告日期"], errors="coerce").dt.date
    big_df["业绩变动幅度"] = pd.to_numeric(big_df["业绩变动幅度"], errors="coerce")
    big_df["预测数值"] = pd.to_numeric(big_df["预测数值"], errors="coerce")
    big_df["上年同期值"] = pd.to_numeric(big_df["上年同期值"], errors="coerce")
    return big_df


def stock_yysj_em(symbol: str = "沪深A股", date: str = "20200331") -> pd.DataFrame:
    """
    东方财富-数据中心-年报季报-预约披露时间
    https://data.eastmoney.com/bbsj/202003/yysj.html
    :param symbol: choice of {'沪深A股', '沪市A股', '科创板', '深市A股', '创业板', '京市A股', 'ST板'}
    :type symbol: str
    :param date: "20190331", "20190630", "20190930", "20191231"; 从 20081231 开始
    :type date: str
    :return: 指定时间的上市公司预约披露时间数据
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "FIRST_APPOINT_DATE,SECURITY_CODE",
        "sortTypes": "1,1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_PUBLIC_BS_APPOIN",
        "columns": "ALL",
        "filter": f"""(SECURITY_TYPE_CODE in ("058001001","058001008"))(TRADE_MARKET_CODE!="069001017")
        (REPORT_DATE='{"-".join([date[:4], date[4:6], date[6:]])}')""",
    }
    if symbol == "沪市A股":
        params.update(
            {
                "filter": f"""(SECURITY_TYPE_CODE in ("058001001","058001008"))
                (TRADE_MARKET_CODE in ("069001001001","069001001003","069001001006"))
                (REPORT_DATE='{"-".join([date[:4], date[4:6], date[6:]])}')"""
            }
        )
    elif symbol == "科创板":
        params.update(
            {
                "filter": f"""(SECURITY_TYPE_CODE in ("058001001","058001008"))(TRADE_MARKET_CODE="069001001006")
                (REPORT_DATE='{"-".join([date[:4], date[4:6], date[6:]])}')"""
            }
        )
    elif symbol == "深市A股":
        params.update(
            {
                "filter": f"""(SECURITY_TYPE_CODE="058001001")(TRADE_MARKET_CODE in
                ("069001002001","069001002002","069001002003","069001002005"))
                (REPORT_DATE='{"-".join([date[:4], date[4:6], date[6:]])}')"""
            }
        )
    elif symbol == "创业板":
        params.update(
            {
                "filter": f"""(SECURITY_TYPE_CODE="058001001")(TRADE_MARKET_CODE="069001002002")
                (REPORT_DATE='{"-".join([date[:4], date[4:6], date[6:]])}')"""
            }
        )
    elif symbol == "京市A股":
        params.update(
            {
                "filter": f"""(TRADE_MARKET_CODE="069001017")
                (REPORT_DATE='{"-".join([date[:4], date[4:6], date[6:]])}')"""
            }
        )
    elif symbol == "ST板":
        params.update(
            {
                "filter": f"""(TRADE_MARKET_CODE in("069001001003","069001002005"))
                (REPORT_DATE='{"-".join([date[:4], date[4:6], date[6:]])}')"""
            }
        )
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update(
            {
                "pageNumber": page,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.reset_index(inplace=True)
    big_df["index"] = range(1, len(big_df) + 1)
    big_df.columns = [
        "序号",
        "股票代码",
        "股票简称",
        "_",
        "_",
        "首次预约时间",
        "一次变更日期",
        "二次变更日期",
        "三次变更日期",
        "实际披露时间",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "股票代码",
            "股票简称",
            "首次预约时间",
            "一次变更日期",
            "二次变更日期",
            "三次变更日期",
            "实际披露时间",
        ]
    ]
    big_df["首次预约时间"] = pd.to_datetime(
        big_df["首次预约时间"], errors="coerce"
    ).dt.date
    big_df["一次变更日期"] = pd.to_datetime(
        big_df["一次变更日期"], errors="coerce"
    ).dt.date
    big_df["二次变更日期"] = pd.to_datetime(
        big_df["二次变更日期"], errors="coerce"
    ).dt.date
    big_df["三次变更日期"] = pd.to_datetime(
        big_df["三次变更日期"], errors="coerce"
    ).dt.date
    big_df["实际披露时间"] = pd.to_datetime(
        big_df["实际披露时间"], errors="coerce"
    ).dt.date
    return big_df


if __name__ == "__main__":
    stock_yjkb_em_df = stock_yjkb_em(date="20200331")
    print(stock_yjkb_em_df)

    stock_yjyg_em_df = stock_yjyg_em(date="20250331")
    print(stock_yjyg_em_df)

    stock_yysj_em_df = stock_yysj_em(symbol="沪深A股", date="20211231")
    print(stock_yysj_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/12/24 16:37
Desc: 东方财富网-数据中心-新股申购-辅导备案信息
https://data.eastmoney.com/xg/ipo/fd.html
"""

import json

import pandas as pd
import requests

from akshare.utils.cons import headers
from akshare.utils.tqdm import get_tqdm


def stock_ipo_tutor_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-新股数据-IPO辅导信息
    https://data.eastmoney.com/xg/ipo/fd.html
    :return: IPO辅导信息
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "RECORD_DATE,TUTOR_OBJECT",
        "sortTypes": "-1,-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_IPO_TUTRECORD",
        "columns": "TUTOR_OBJECT,ORG_CODE,TUTOR_ORG_CODE,TUTOR_ORG,TUTOR_PROCESS_STATE,REPORT_TYPE,"
        "DISPATCH_ORG,REPORT_TITLE,RECORD_DATE",
        "source": "WEB",
        "client": "WEB",
    }

    def parse_jsonp(text: str):
        """安全解析 JSONP 响应"""
        # 找到第一个 { 和最后一个 } 之间的内容
        start = text.find("{")
        end = text.rfind("}")
        if start == -1 or end == -1 or end <= start:
            raise ValueError(f"无法找到 JSON 对象: {text[:100]}...")
        json_str = text[start : end + 1]
        return json.loads(json_str)

    # 首次请求获取总页数
    r = requests.get(url, params=params, headers=headers)
    r.raise_for_status()  # 如果请求失败，抛出异常
    data_json = parse_jsonp(r.text)

    if "result" not in data_json or "pages" not in data_json["result"]:
        raise ValueError("响应格式异常，未找到 result.pages")

    page_num = data_json["result"]["pages"]
    big_df = pd.DataFrame()

    tqdm = get_tqdm()
    for page in tqdm(range(1, page_num + 1), leave=False):
        params["pageNumber"] = str(page)
        r = requests.get(url, params=params, headers=headers)
        r.raise_for_status()
        try:
            data_json = parse_jsonp(r.text)
            temp_df = pd.DataFrame(data_json["result"]["data"])
            big_df = pd.concat([big_df, temp_df], ignore_index=True)
        except Exception as e:
            print(f"第 {page} 页解析失败: {e}")
            continue

    big_df.reset_index(drop=True, inplace=True)
    big_df.insert(0, "序号", big_df.index + 1)

    big_df.rename(
        columns={
            "TUTOR_OBJECT": "企业名称",
            "TUTOR_ORG": "辅导机构",
            "TUTOR_PROCESS_STATE": "辅导状态",
            "REPORT_TYPE": "报告类型",
            "DISPATCH_ORG": "派出机构",
            "REPORT_TITLE": "报告标题",
            "RECORD_DATE": "备案日期",
        },
        inplace=True,
    )

    big_df = big_df[
        [
            "序号",
            "企业名称",
            "辅导机构",
            "辅导状态",
            "报告类型",
            "派出机构",
            "报告标题",
            "备案日期",
        ]
    ]

    big_df["备案日期"] = pd.to_datetime(big_df["备案日期"], errors="coerce").dt.date

    return big_df


if __name__ == "__main__":
    stock_ipo_tutor_em_df = stock_ipo_tutor_em()
    print(stock_ipo_tutor_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/4/10 18:00
Desc: 接口测试文件
"""

from akshare.pro.data_pro import pro_api

pro = pro_api()

# 商品-持仓数据-多头龙虎榜
variety_positions_longs_df = pro.variety_positions(
    fields="longs", code="rb1810", date="2018-08-08"
)
print(variety_positions_longs_df)

# 商品-持仓数据-空头龙虎榜
variety_positions_shorts_df = pro.variety_positions(
    fields="shorts", code="rb1810", date="2018-08-08"
)
print(variety_positions_shorts_df)

# 商品-合约行情数据
variety_quotes_df = pro.variety_quotes(code="rb1810", date="2018-08-08")
print(variety_quotes_df)

# 商品-商品沉淀资金数据接口
variety_money_df = pro.variety_money(symbol="RB", date="2018-08-08")
print(variety_money_df)

# 商品-合约多空比数据
variety_bbr_df = pro.variety_bbr(code="rb1810", date="2018-08-08")
print(variety_bbr_df)

# 商品-合约净持仓保证金变化数据
variety_net_money_chge_df = pro.variety_net_money_chge(code="rb1810", date="2018-08-08")
print(variety_net_money_chge_df)

# 商品-合约净持仓保证金数据
variety_net_money_df = pro.variety_net_money(code="rb1810", date="2018-08-08")
print(variety_net_money_df)

# 商品-合约总持仓保证金数据
variety_total_money_df = pro.variety_total_money(code="rb1810", date="2018-08-08")
print(variety_total_money_df)

# 商品-商品的席位盈亏数据
variety_profit_df = pro.variety_profit(
    symbol="RB", start_date="2018-02-08", end_date="2018-08-08"
)
print(variety_profit_df)

# 商品-自研指标数据
variety_strategies_df = pro.variety_strategies(code="rb1810", date="2018-08-08")
print(variety_strategies_df)

# 商品-龙虎比排行数据-多头排行
variety_longhu_long_top_df = pro.variety_longhu_top(fields="long", date="2018-08-08")
print(variety_longhu_long_top_df)

# 商品-龙虎比排行数据-空头排行
variety_longhu_short_top_df = pro.variety_longhu_top(fields="short", date="2018-08-08")
print(variety_longhu_short_top_df)

# 商品-牛熊线排行数据-多头排行
variety_niuxiong_long_top_df = pro.variety_niuxiong_top(
    fields="long", date="2018-08-08"
)
print(variety_niuxiong_long_top_df)

# 商品-牛熊线排行数据-空头排行
variety_niuxiong_short_top_df = pro.variety_niuxiong_top(
    fields="short", date="2018-08-08"
)
print(variety_niuxiong_short_top_df)

# 商品-商品相关研报数据
variety_reports_df = pro.variety_reports(csymbolode="RB", date="2018-08-08")
print(variety_reports_df)

# 商品-商品列表数据
variety_all_df = pro.variety_all()
print(variety_all_df)


# 席位-商品净持仓数据
variety_net_positions_df = pro.variety_net_positions(
    symbol="RB", broker="永安期货", date="2018-08-08"
)
print(variety_net_positions_df)

# 席位-席位持仓数据
broker_positions_df = pro.broker_positions(broker="永安期货", date="2018-08-08")
print(broker_positions_df)

# 席位-席位盈亏数据
broker_calendar_df = pro.broker_calendar(
    broker="永安期货", start_date="2018-07-08", end_date="2018-08-08"
)
print(broker_calendar_df)

# 席位-席位每日大资金流动数据
broker_flow_df = pro.broker_flow(broker="永安期货", date="2018-08-08", offset="1000000")
print(broker_flow_df)

# 席位-席位多空比数据
broker_bbr_df = pro.broker_bbr(broker="永安期货", date="2018-08-08")
print(broker_bbr_df)

# 席位-席位净持仓保证金变化数据
broker_net_money_chge_df = pro.broker_net_money_chge(
    broker="永安期货", date="2018-08-08"
)
print(broker_net_money_chge_df)

# 席位-席位净持仓保证金数据
broker_net_money_df = pro.broker_net_money(broker="永安期货", date="2018-08-08")
print(broker_net_money_df)

# 席位-席位总持仓保证金数据
broker_total_money_df = pro.broker_total_money(broker="永安期货", date="2018-08-08")
print(broker_total_money_df)

# 席位-席位的商品盈亏数据
broker_profit_df = pro.broker_profit(
    broker="永安期货", start_date="2018-07-08", end_date="2018-08-08"
)
print(broker_profit_df)

# 席位-席位盈利排行
broker_in_profit_list_df = pro.broker_in_profit_list(
    start_date="2018-07-08", end_date="2018-08-08", count="10"
)
print(broker_in_profit_list_df)

# 席位-席位亏损排行
broker_in_loss_list_df = pro.broker_in_loss_list(
    start_date="2018-07-08", end_date="2018-08-08", count="10"
)
print(broker_in_loss_list_df)

# 席位-所有席位数据
broker_all_df = pro.broker_all(offset_days="365")
print(broker_all_df)

# 席位-建仓过程
broker_positions_process_df = pro.broker_positions_process(
    broker="永安期货", code="rb2010", start_date="2020-02-03", end_date="2020-06-03"
)
print(broker_positions_process_df)

# 席位-席位对对碰
broker_pk_df = pro.broker_pk(broker1="永安期货", broker2="兴证期货", symbol="RB")
print(broker_pk_df)


# 指数-指数信息
index_info_df = pro.index_info(index_id="index0070c0eb-93ba-2da9-6633-fa70cb90e959")
print(index_info_df)

# 指数-指数权重数据
index_weights_df = pro.index_weights(
    index_id="index0070c0eb-93ba-2da9-6633-fa70cb90e959", date="2018-08-08"
)
print(index_weights_df)

# 指数-指数行情数据
index_quotes_df = pro.index_quotes(
    index_id="index0070c0eb-93ba-2da9-6633-fa70cb90e959", date="2018-08-08"
)
print(index_quotes_df)

# 指数-指数沉淀资金数据
index_money_df = pro.index_money(
    index_id="index0070c0eb-93ba-2da9-6633-fa70cb90e959", date="2018-08-08"
)
print(index_money_df)

# 指数-公共指数列表
index_official_df = pro.index_official()
print(index_official_df)

# 指数-个人指数列表
index_mine_df = pro.index_mine()
print(index_mine_df)

# 指数-指数资金动向
index_trend_df = pro.index_trend(
    index_id="index0070c0eb-93ba-2da9-6633-fa70cb90e959", date="2018-08-08"
)
print(index_trend_df)

# 指数-指数的席位盈亏数据
index_profit_df = pro.index_profit(
    index_id="index0070c0eb-93ba-2da9-6633-fa70cb90e959",
    start_date="2018-07-08",
    end_date="2018-08-08",
)
print(index_profit_df)

# 基本面-基差
basis_df = pro.basis(variety="RB", date="2018-08-08")
print(basis_df)

# 基本面-期限结构
term_structure_df = pro.term_structure(variety="RB", date="2018-08-08")
print(term_structure_df)

# 基本面-库存数据
inventory_df = pro.inventory(variety="RB", date="2018-08-08")
print(inventory_df)

# 基本面-利润数据
profit_df = pro.profit(variety="RB", date="2019-12-12")
print(profit_df)

# 基本面-现货贸易商报价
trader_prices_df = pro.trader_prices(variety="RB", date="2020-03-30")
print(trader_prices_df)

# 基本面-跨期套利数据
intertemporal_arbitrage_df = pro.intertemporal_arbitrage(
    variety="RB", code1="01", code2="05", date="2018-08-08"
)
print(intertemporal_arbitrage_df)

# 基本面-自由价差数据
free_spread_df = pro.free_spread(
    variety1="RB", code1="01", variety2="HC", code2="01", date="2018-08-08"
)
print(free_spread_df)

# 基本面-自由价比数据
free_ratio_df = pro.free_ratio(
    variety1="RB", code1="01", variety2="HC", code2="01", date="2018-08-08"
)
print(free_ratio_df)

# 基本面-仓单数据
warehouse_receipt_df = pro.warehouse_receipt(variety="RB", date="2018-08-08")
print(warehouse_receipt_df)

# 基本面-仓单汇总数据
warehouse_receipt_sum_df = pro.warehouse_receipt(date="2018-08-08")
print(warehouse_receipt_sum_df)

# 基本面-虚实盘比数据
virtual_real_df = pro.virtual_real(variety="RB", code="10", date="2018-08-08")
print(virtual_real_df)


# 工具-龙虎牛熊多头合约池
long_pool_df = pro.long_pool(date="2018-08-08")
print(long_pool_df)

# 工具-龙虎牛熊空头合约池
short_pool_df = pro.short_pool(date="2018-08-08")
print(short_pool_df)


# 资金-每日净流多列表(商品)
commodity_flow_long_df = pro.commodity_flow_long(date="2018-08-08")
print(commodity_flow_long_df)

# 资金-每日净流空列表(商品)
commodity_flow_short_df = pro.commodity_flow_short(date="2018-08-08")
print(commodity_flow_short_df)

# 资金-每日净流多列表(指数)
stock_flow_long_df = pro.stock_flow_long(date="2018-08-08")
print(stock_flow_long_df)

# 资金-每日净流空列表(指数)
stock_flow_short_df = pro.stock_flow_short(date="2018-08-08")
print(stock_flow_short_df)

# 资金-每日商品保证金沉淀变化
money_in_out_df = pro.money_in_out(date="2018-08-08")
print(money_in_out_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/7/15 16:30
Desc: 新浪财经-ESG评级中心
https://finance.sina.com.cn/esg/
"""

import math
from akshare.utils.tqdm import get_tqdm

import pandas as pd
import requests


def stock_esg_msci_sina() -> pd.DataFrame:
    """
    新浪财经-ESG评级中心-ESG评级-MSCI
    https://finance.sina.com.cn/esg/grade.shtml
    :return: MSCI
    :rtype: pandas.DataFrame
    """
    url = "https://global.finance.sina.com.cn/api/openapi.php/EsgService.getMsciEsgStocks?p=1&num=100"
    r = requests.get(url)
    data_json = r.json()
    page_num = math.ceil(int(data_json["result"]["data"]["total"]) / 100)
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, page_num + 1), leave=False):
        headers = {
            "Referer": "https://finance.sina.com.cn/",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/123.0.0.0 Safari/537.36",
        }
        url = f"https://global.finance.sina.com.cn/api/openapi.php/EsgService.getMsciEsgStocks?p={page}&num=100"
        r = requests.get(url, headers=headers)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.rename(
        columns={
            "agency_id": "-",
            "agency_name": "评级机构",
            "symbol": "股票代码",
            "delist": "-",
            "comp_code": "-",
            "name": "股票名称",
            "market": "交易市场",
            "industry_code": "行业代码",
            "industry_name": "行业名称",
            "sw1_code": "-",
            "sw1_name": "-",
            "sw2_code": "-",
            "sw2_name": "-",
            "sw3_code": "-",
            "sw3_name": "-",
            "hs1_code": "-",
            "hs1_name": "-",
            "hs2_code": "-",
            "hs2_name": "-",
            "hs3_code": "-",
            "hs3_name": "-",
            "factset_sector_code": "-",
            "factset_sector_name": "-",
            "factset_industry_code": "-",
            "factset_industry_name": "-",
            "date": "-",
            "quarter_date": "评级日期",
            "grade": "ESG等级",
            "score": "-",
            "env_score": "环境总评",
            "env_grade": "-",
            "social_score": "社会责任总评",
            "social_grade": "-",
            "governance_score": "治理总评",
            "governance_grade": "-",
            "change_status": "-",
            "updated_time": "更新时间",
            "created_time": "创建时间",
            "esg_rating": "ESG评分",
        },
        inplace=True,
    )
    big_df = big_df[
        [
            "股票代码",
            "ESG评分",
            "环境总评",
            "社会责任总评",
            "治理总评",
            "评级日期",
            "交易市场",
        ]
    ]
    big_df["评级日期"] = pd.to_datetime(big_df["评级日期"], errors="coerce").dt.date
    big_df["环境总评"] = pd.to_numeric(big_df["环境总评"], errors="coerce")
    big_df["社会责任总评"] = pd.to_numeric(big_df["社会责任总评"], errors="coerce")
    big_df["治理总评"] = pd.to_numeric(big_df["治理总评"], errors="coerce")
    return big_df


def stock_esg_rft_sina() -> pd.DataFrame:
    """
    新浪财经-ESG评级中心-ESG评级-路孚特
    https://finance.sina.com.cn/esg/grade.shtml
    :return: 路孚特
    :rtype: pandas.DataFrame
    """
    url = "https://global.finance.sina.com.cn/api/openapi.php/EsgService.getRftEsgStocks?p=1&num=20000"
    r = requests.get(url)
    data_json = r.json()
    big_df = pd.DataFrame(data_json["result"]["data"]["data"])
    big_df.rename(
        columns={
            "symbol": "股票代码",
            "esg_score": "ESG评分",
            "esg_score_date": "ESG评分日期",
            "env_score": "环境总评",
            "env_score_date": "环境总评日期",
            "social_score": "社会责任总评",
            "social_score_date": "社会责任总评日期",
            "governance_score": "治理总评",
            "governance_score_date": "治理总评日期",
            "zy_score": "争议总评",
            "zy_score_date": "争议总评日期",
            "industry": "行业",
            "exchange": "交易所",
        },
        inplace=True,
    )
    big_df = big_df[
        [
            "股票代码",
            "ESG评分",
            "ESG评分日期",
            "环境总评",
            "环境总评日期",
            "社会责任总评",
            "社会责任总评日期",
            "治理总评",
            "治理总评日期",
            "争议总评",
            "争议总评日期",
            "行业",
            "交易所",
        ]
    ]
    big_df["ESG评分日期"] = pd.to_datetime(
        big_df["ESG评分日期"], errors="coerce"
    ).dt.date
    big_df["环境总评日期"] = pd.to_datetime(
        big_df["环境总评日期"], errors="coerce"
    ).dt.date
    big_df["社会责任总评日期"] = pd.to_datetime(
        big_df["社会责任总评日期"], errors="coerce"
    ).dt.date
    big_df["治理总评日期"] = pd.to_datetime(
        big_df["治理总评日期"], errors="coerce"
    ).dt.date
    big_df["争议总评日期"] = pd.to_datetime(
        big_df["争议总评日期"], errors="coerce"
    ).dt.date
    return big_df


def stock_esg_rate_sina() -> pd.DataFrame:
    """
    新浪财经-ESG评级中心-ESG评级-ESG评级数据
    https://finance.sina.com.cn/esg/grade.shtml
    :return: ESG评级数据
    :rtype: pandas.DataFrame
    """
    url = "https://global.finance.sina.com.cn/api/openapi.php/EsgService.getEsgStocks?page=1&num=200"
    r = requests.get(url)
    data_json = r.json()
    page_num = math.ceil(int(data_json["result"]["data"]["info"]["total"]) / 200)
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, page_num + 1), leave=False):
        url = f"https://global.finance.sina.com.cn/api/openapi.php/EsgService.getEsgStocks?page={page}&num=200"
        r = requests.get(url)
        data_json = r.json()
        stock_num = len(data_json["result"]["data"]["info"]["stocks"])
        for num in range(stock_num):
            temp_df = pd.DataFrame(
                data_json["result"]["data"]["info"]["stocks"][num]["esg_info"]
            )
            temp_df["symbol"] = data_json["result"]["data"]["info"]["stocks"][num][
                "symbol"
            ]
            temp_df["market"] = data_json["result"]["data"]["info"]["stocks"][num][
                "market"
            ]
            big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.rename(
        columns={
            "symbol": "成分股代码",
            "agency_name": "评级机构",
            "esg_score": "评级",
            "esg_dt": "评级季度",
            "remark": "标识",
            "market": "交易市场",
        },
        inplace=True,
    )
    big_df = big_df[
        [
            "成分股代码",
            "评级机构",
            "评级",
            "评级季度",
            "标识",
            "交易市场",
        ]
    ]
    return big_df


def stock_esg_zd_sina() -> pd.DataFrame:
    """
    新浪财经-ESG评级中心-ESG评级-秩鼎
    https://finance.sina.com.cn/esg/grade.shtml
    :return: 秩鼎
    :rtype: pandas.DataFrame
    """
    url = "https://global.finance.sina.com.cn/api/openapi.php/EsgService.getZdEsgStocks"
    params = {"p": "1", "num": "100"}
    r = requests.get(url, params=params)
    data_json = r.json()
    tqdm = get_tqdm()
    total_page = math.ceil(int(data_json["result"]["data"]["total"]) / 100)
    temp_list = []
    for page in tqdm(range(1, total_page + 1), leave=False):
        params = {"p": str(page), "num": "100"}
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"]["data"])
        temp_list.append(temp_df)
    big_df = pd.concat(temp_list, ignore_index=True)
    big_df.rename(
        columns={
            "ticker": "股票代码",
            "esg_score": "ESG评分",
            "report_date": "评分日期",
            "environmental_score": "环境总评",
            "social_score": "社会责任总评",
            "governance_score": "治理总评",
        },
        inplace=True,
    )
    big_df = big_df[
        [
            "股票代码",
            "ESG评分",
            "环境总评",
            "社会责任总评",
            "治理总评",
            "评分日期",
        ]
    ]
    big_df["评分日期"] = pd.to_datetime(big_df["评分日期"], errors="coerce").dt.date
    return big_df


def stock_esg_hz_sina() -> pd.DataFrame:
    """
    新浪财经-ESG评级中心-ESG评级-华证指数
    https://finance.sina.com.cn/esg/grade.shtml
    :return: 华证指数
    :rtype: pandas.DataFrame
    """
    url = "https://global.finance.sina.com.cn/api/openapi.php/EsgService.getHzEsgStocks"
    params = {"p": 1, "num": "100"}
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = math.ceil(int(data_json["result"]["data"]["total"]) / 100)
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params = {"p": str(page), "num": "100"}
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.rename(
        columns={
            "date": "日期",
            "symbol": "股票代码",
            "market": "交易市场",
            "name": "股票名称",
            "esg_score": "ESG评分",
            "esg_score_grade": "ESG等级",
            "e_score": "环境",
            "e_score_grade": "环境等级",
            "s_score": "社会",
            "s_score_grade": "社会等级",
            "g_score": "公司治理",
            "g_score_grade": "公司治理等级",
        },
        inplace=True,
    )
    big_df = big_df[
        [
            "日期",
            "股票代码",
            "交易市场",
            "股票名称",
            "ESG评分",
            "ESG等级",
            "环境",
            "环境等级",
            "社会",
            "社会等级",
            "公司治理",
            "公司治理等级",
        ]
    ]
    big_df["日期"] = pd.to_datetime(big_df["日期"], errors="coerce").dt.date
    big_df["ESG评分"] = pd.to_numeric(big_df["ESG评分"], errors="coerce")
    big_df["环境"] = pd.to_numeric(big_df["环境"], errors="coerce")
    big_df["社会"] = pd.to_numeric(big_df["社会"], errors="coerce")
    big_df["公司治理"] = pd.to_numeric(big_df["公司治理"], errors="coerce")
    return big_df


if __name__ == "__main__":
    stock_esg_msci_sina_df = stock_esg_msci_sina()
    print(stock_esg_msci_sina_df)

    stock_esg_rft_sina_df = stock_esg_rft_sina()
    print(stock_esg_rft_sina_df)

    stock_esg_rate_sina_df = stock_esg_rate_sina()
    print(stock_esg_rate_sina_df)

    stock_esg_zd_sina_df = stock_esg_zd_sina()
    print(stock_esg_zd_sina_df)

    stock_esg_hz_sina_df = stock_esg_hz_sina()
    print(stock_esg_hz_sina_df)
